==> netsa-io-silk_2.12-1.5.1-sources.jar: io/unpackers/FT_RWAUGWEB_v1.scala <==
// Copyright 2015-2022 by Carnegie Mellon University
// See license information in LICENSE.txt

package org.cert.netsa.io.silk
package io
package unpackers

import org.cert.netsa.data.net.{IPv4Address, SNMPInterface}

import BufferUtil._

/*
**  RWAUGWEB VERSION 1
**  RWAUGWEB VERSION 2
**  RWAUGWEB VERSION 3
**
**  in the following: EXPANDED == ((tcp_state & SK_TCPSTATE_EXPANDED) ? 1 : 0)
**
**    uint32_t      sIP;             //  0- 3  Source IP
**    uint32_t      dIP;             //  4- 7  Destination IP
**
**    uint32_t      pkts_stime;      //  8-11
**    // uint32_t     pkts      :20; //        Count of packets
**    // uint32_t     sTime     :12; //        Start time--offset from hour
**
**    uint32_t      bbe;             // 12-15
**    // uint32_t     bPPkt     :14; //        Whole bytes-per-packet
**    // uint32_t     bPPFrac   : 6; //        Fractional bytes-per-packet
**    // uint32_t     elapsed   :12; //        Duration of flow
**
**    uint32_t      msec_prt_flags   // 16-19
**    // uint32_t     sTime_msec:10; //        Fractional sTime (millisec)
**    // uint32_t     elaps_msec:10; //        Fractional elapsed (millisec)
**    // uint32_t     pflag     : 1; //        'pkts' requires multiplier?
**    // uint32_t     srcIsSrv  : 1; //        1 if srv_port is src; 0 if dest
**    // uint32_t     srv_port  : 2; //        server port: 0=80; 1=443; 2=8080
**    // uint32_t     a_1_flags : 8; //        EXPANDED==0: TCP flags/All pkts
**                                   //        EXPANDED==1: TCP flags/1st pkt
**
**    uint16_t      clnt_port;       // 20-21  Non-Web Port
**
**    uint16_t      application;     // 22-23  Type of traffic
**
**    uint8_t       tcp_state;       // 24     TCP state machine info
**    uint8_t       rest_flags;      // 25     EXPANDED==0: Empty
**                                   //        EXPANDED==1: TCPflags/rest pkts
**
**
**  26 bytes on disk.
*/

private[silk] object FT_RWAUGWEB_v1 extends Unpacker {
  val recordLength = 26
  def unpack(
    buffer: Array[Byte], offset: Int, swap: Boolean, header: Header): RWRec =
  {
    val sIP = getIPv4Address(buffer, offset, swap)
    val dIP = getIPv4Address(buffer, offset + 4, swap)
    val (startTime, packets, bytes, elapsed, inProtocol, inFlags) =
      getTimeBytesPacketsFlags(
        buffer, offset + 8, offset + 12, offset + 16, false, swap, header)
    val msecPrtFlags = getInt32(buffer, offset + 16, swap)
    val a1Flags = msecPrtFlags.toByte

    val clientPort = getPort(buffer, offset + 20, swap)
    val serverPort = decodeWWWPort((msecPrtFlags >> 8) & 0x00000003)
    val sourceIsServer = (msecPrtFlags & 0x00000400) != 0
    val (sPort, dPort) = if ( sourceIsServer ) {
      (serverPort, clientPort)
    } else {
      (clientPort, serverPort)
    }

    val tcpState = getTCPState(buffer, offset + 24, swap)
    val inRestFlags = getInt8(buffer, offset + 25, swap)
    val (protocol, flags, initFlags, restFlags) = unpackProtoFlags(
      true, a1Flags, tcpState, inRestFlags)
    val application = getApplication(buffer, offset + 22, swap)
    val flowType = header.packedFlowTypeId
    val sensor = header.packedSensorId
    val memo: Short = 0
    val input = SNMPInterface(0)
    val output = SNMPInterface(0)
    val nhIP = IPv4Address(0)
    RWRec(
      startTime, elapsed, sPort, dPort, protocol, flowType, sensor,
      flags, initFlags, restFlags, tcpState, application, memo, input,
      output, packets, bytes, sIP, dIP, nhIP)
  }
}

// @LICENSE_FOOTER@
//
// Copyright 2015-2022 Carnegie Mellon University. All Rights Reserved.
//
// This material is based upon work funded and supported by the
// Department of Defense and Department of Homeland Security under
// Contract No. FA8702-15-D-0002 with Carnegie Mellon University for the
// operation of the Software Engineering Institute, a federally funded
// research and development center sponsored by the United States
// Department of Defense. The U.S. Government has license rights in this
// software pursuant to DFARS 252.227.7014.
//
// NO WARRANTY. THIS CARNEGIE MELLON UNIVERSITY AND SOFTWARE ENGINEERING
// INSTITUTE MATERIAL IS FURNISHED ON AN "AS-IS" BASIS. CARNEGIE MELLON
// UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER EXPRESSED OR
// IMPLIED, AS TO ANY MATTER INCLUDING, BUT NOT LIMITED TO, WARRANTY OF
// FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY, OR RESULTS
// OBTAINED FROM USE OF THE MATERIAL. CARNEGIE MELLON UNIVERSITY DOES NOT
// MAKE ANY WARRANTY OF ANY KIND WITH RESPECT TO FREEDOM FROM PATENT,
// TRADEMARK, OR COPYRIGHT INFRINGEMENT.
//
// Released under a GNU GPL 2.0-style license, please see LICENSE.txt or
// contact permission@sei.cmu.edu for full terms.
//
// [DISTRIBUTION STATEMENT A] This material has been approved for public
// release and unlimited distribution. Please see Copyright notice for
// non-US Government use and distribution.
//
// Carnegie Mellon(R) and CERT(R) are registered in the U.S. Patent and
// Trademark Office by Carnegie Mellon University.
//
// This software includes and/or makes use of third party software each
// subject to its own license as detailed in LICENSE-thirdparty.tx
//
// DM20-1143

==> netsa-io-silk_2.12-1.6.0-sources.jar: io/unpackers/FT_RWAUGWEB_v1.scala <==
// Copyright 2015-2022 by Carnegie Mellon University
// See license information in LICENSE.txt

package org.cert.netsa.io.silk
package io
package unpackers

import org.cert.netsa.data.net.{IPv4Address, SNMPInterface}

import BufferUtil._

/*
**  RWAUGWEB VERSION 1
**  RWAUGWEB VERSION 2
**  RWAUGWEB VERSION 3
**
**  in the following: EXPANDED == ((tcp_state & SK_TCPSTATE_EXPANDED) ? 1 : 0)
**
**    uint32_t      sIP;             //  0- 3  Source IP
**    uint32_t      dIP;             //  4- 7  Destination IP
**
**    uint32_t      pkts_stime;      //  8-11
**    // uint32_t     pkts      :20; //        Count of packets
**    // uint32_t     sTime     :12; //        Start time--offset from hour
**
**    uint32_t      bbe;             // 12-15
**    // uint32_t     bPPkt     :14; //        Whole bytes-per-packet
**    // uint32_t     bPPFrac   : 6; //        Fractional bytes-per-packet
**    // uint32_t     elapsed   :12; //        Duration of flow
**
**    uint32_t      msec_prt_flags   // 16-19
**    // uint32_t     sTime_msec:10; //        Fractional sTime (millisec)
**    // uint32_t     elaps_msec:10; //        Fractional elapsed (millisec)
**    // uint32_t     pflag     : 1; //        'pkts' requires multiplier?
**    // uint32_t     srcIsSrv  : 1; //        1 if srv_port is src; 0 if dest
**    // uint32_t     srv_port  : 2; //        server port: 0=80; 1=443; 2=8080
**    // uint32_t     a_1_flags : 8; //        EXPANDED==0: TCP flags/All pkts
**                                   //        EXPANDED==1: TCP flags/1st pkt
**
**    uint16_t      clnt_port;       // 20-21  Non-Web Port
**
**    uint16_t      application;     // 22-23  Type of traffic
**
**    uint8_t       tcp_state;       // 24     TCP state machine info
**    uint8_t       rest_flags;      // 25     EXPANDED==0: Empty
**                                   //        EXPANDED==1: TCPflags/rest pkts
**
**
**  26 bytes on disk.
*/

private[silk] object FT_RWAUGWEB_v1 extends Unpacker {
  val recordLength = 26
  def unpack(
    buffer: Array[Byte], offset: Int, swap: Boolean, header: Header): RWRec =
  {
    val sIP = getIPv4Address(buffer, offset, swap)
    val dIP = getIPv4Address(buffer, offset + 4, swap)
    val (startTime, packets, bytes, elapsed, _, _) =
      getTimeBytesPacketsFlags(
        buffer, offset + 8, offset + 12, offset + 16, false, swap, header)
    val msecPrtFlags = getInt32(buffer, offset + 16, swap)
    val a1Flags = msecPrtFlags.toByte

    val clientPort = getPort(buffer, offset + 20, swap)
    val serverPort = decodeWWWPort((msecPrtFlags >> 8) & 0x00000003)
    val sourceIsServer = (msecPrtFlags & 0x00000400) != 0
    val (sPort, dPort) = if ( sourceIsServer ) {
      (serverPort, clientPort)
    } else {
      (clientPort, serverPort)
    }

    val tcpState = getTCPState(buffer, offset + 24, swap)
    val inRestFlags = getInt8(buffer, offset + 25, swap)
    val (protocol, flags, initFlags, restFlags) = unpackProtoFlags(
      true, a1Flags, tcpState, inRestFlags)
    val application = getApplication(buffer, offset + 22, swap)
    val flowType = header.packedFlowTypeId
    val sensor = header.packedSensorId
    val memo: Short = 0
    val input = SNMPInterface(0)
    val output = SNMPInterface(0)
    val nhIP = IPv4Address(0)
    RWRec(
      startTime, elapsed, sPort, dPort, protocol, flowType, sensor,
      flags, initFlags, restFlags, tcpState, application, memo, input,
      output, packets, bytes, sIP, dIP, nhIP)
  }
}

// @LICENSE_FOOTER@
//
// Copyright 2015-2022 Carnegie Mellon University. All Rights Reserved.
//
// This material is based upon work funded and supported by the
// Department of Defense and Department of Homeland Security under
// Contract No. FA8702-15-D-0002 with Carnegie Mellon University for the
// operation of the Software Engineering Institute, a federally funded
// research and development center sponsored by the United States
// Department of Defense. The U.S. Government has license rights in this
// software pursuant to DFARS 252.227.7014.
//
// NO WARRANTY. THIS CARNEGIE MELLON UNIVERSITY AND SOFTWARE ENGINEERING
// INSTITUTE MATERIAL IS FURNISHED ON AN "AS-IS" BASIS. CARNEGIE MELLON
// UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER EXPRESSED OR
// IMPLIED, AS TO ANY MATTER INCLUDING, BUT NOT LIMITED TO, WARRANTY OF
// FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY, OR RESULTS
// OBTAINED FROM USE OF THE MATERIAL. CARNEGIE MELLON UNIVERSITY DOES NOT
// MAKE ANY WARRANTY OF ANY KIND WITH RESPECT TO FREEDOM FROM PATENT,
// TRADEMARK, OR COPYRIGHT INFRINGEMENT.
//
// Released under a GNU GPL 2.0-style license, please see LICENSE.txt or
// contact permission@sei.cmu.edu for full terms.
//
// [DISTRIBUTION STATEMENT A] This material has been approved for public
// release and unlimited distribution. Please see Copyright notice for
// non-US Government use and distribution.
//
// Carnegie Mellon(R) and CERT(R) are registered in the U.S. Patent and
// Trademark Office by Carnegie Mellon University.
//
// This software includes and/or makes use of third party software each
// subject to its own license as detailed in LICENSE-thirdparty.tx
//
// DM20-1143
